/**
 * This is a base class used only to wrap a few required attributes away from each custom exception
 */
declare class BaseException extends Error {
    readonly _e = true;
    readonly key: string;
    constructor(msg: string, key: string);
}
export declare class Exception_OutOfBounds extends BaseException {
    static defaultMessage: string;
    static key: string;
    constructor(msg?: string);
}
export declare class Exception_FindReplaceIllegalAction extends BaseException {
    static defaultMessage: string;
    static key: string;
    constructor(msg?: string);
}
export {};
/**
 * This is ~~black magic~~ a helper type for declaring static variables to be present on the type of a static input.
 *
 * Say you want to put in the class declaration and not an instance of the class, this interface wraps this
 * and provides a required typed constructor.
 *
 * Example:
 * ```typescript
 *
 * const myMethod = (input: ClassBuilder) => {
 *    console.log(input.key)
 * }
 *
 * myMethod(BaseError) // Gives type error `Property 'key' is missing in type 'typeof BaseError' but required in type 'ClassBuilder<BaseError>'`
 *
 * myMethod(OutOfBounds) // OK
 * ```
 */
/**
 * Assert and type guard input error to be one of the custom Error types.
 * The asserted type is derived from the second parameter.
 *
 * Example:
 * ```typescript
 * try { ... } catch (err) {
 *   assertErrorType(err, Exception_FindReplaceIllegalAction);
 *   // err is now typed as Exception_FindReplaceIllegalAction
 * }
 * ```
 *
 * @param err instance of the error to assert and type guard
 * @param errorClass Static class input of a custom Error
 */
/**
 * Custom type guard of the exception classes.
 *
 * Example:
 * ```typescript
 * try { ... } catch (err) {
 *   if (isErrorType(err, Exception_FindReplaceIllegalAction)) {
 *      // err is now typed as Exception_FindReplaceIllegalAction
 *   }
 * }
 * ```
 *
 * @param err instance of the error to type guard
 * @param errorClass Static class input of a custom Error
 */
